#!/usr/bin/env python3.5

import serial
import time
import sys
import subprocess
import os.path


def sendSerial(toSend, more_timeout=0.0):
    toSend = toSend + '\r'
    ser.write(toSend.encode())
    time.sleep(int(more_timeout))
    print(ser.read(1024).decode('utf-8').replace('\r\n', ' '), flush=True)


def runMult():
    sendSerial('mult 1 16384')
    sendSerial('mult 2 16384')
    sendSerial('mult 3 48000')
    sendSerial('mult 1 32768')
    sendSerial('mult 2 32768')
    sendSerial('mult 3 32768')
    sendSerial('mult') 

    
def runSLED():
    print('=============  SLED ==============')
    sendSerial('mult')
    sendSerial('psat 9000000')
    IFpower = -99.
    found = 0    
    for i in range(int(limits[0]), int(limits[1]), 2):
        outputStr = subprocess.run(
            ['ssh', '192.168.1.61', '/home/ckulesa/src/run_usbpowermeter.pl'], stdout=subprocess.PIPE).stdout.decode('utf-8')
        print('IF Pwr at PSAT = ', i, ' = ', float(outputStr))
        if(float(outputStr) < (IFpower - 0.1)):
            print('SLED IF peak detected')
            sendSerial('mult')
            found = 1
            break
        IFpower = float(outputStr)
        PSAT = i * 100000
        cmdStr = "psat {}".format(PSAT)
        sendSerial(cmdStr)
        sendSerial('psat')
    if(found == 0):
        print('WARNING: SLED run completed with NO peak found')


def runQCL():
    print('================  QCL ==================')
    sendSerial('coil 20000')
    sendSerial('coil 45678')
    sendSerial('mode 1')
    sendSerial('qcl on', 1)
    for i in range(1, 5):
        cmdStr = "qcl {} 25000".format(i)
        sendSerial(cmdStr)
    sendSerial('qcl 2 50000')  # leave one at full power
    sendSerial('qcl')

    
def runIsense():
    print('============ MBB Isense ==============')
    sendSerial('imon')
    for i in [1, 2, 3, 5, 6]:
        cmdStr = "offset {} 4000".format(i)
        sendSerial(cmdStr)
    sendSerial('imon')


def powerDown(wait):
    print('=========== Prep for end of cycle ==============')
    time.sleep(int(wait))
    # record current state
    sendSerial('volts', 1)
    sendSerial('dcdc 8 1')
    # ramp down PSAT but leave DCDC on
    if(noLO == 0):
        PSAT = int(limits[0]) * 100000
        cmdStr = "psat {}".format(PSAT)
        sendSerial(cmdStr)
        sendSerial('psat 8000000')
    sendSerial('psat 1')

    for i in [1, 2, 3, 5, 6]:
        cmdStr = "offset {} 0".format(i)
        sendSerial(cmdStr)

    if(cold == 0):
        for i in range(1, 5):
            cmdStr = "qcl {} 20000".format(i)
            sendSerial(cmdStr)
    else:  
        sendSerial('dcdc 2 1')  # disable hittite synth +12
        sendSerial('dcdc 6 1')  # disable hittite synth +5.5
        for i in range(1, 5):
            cmdStr = "qcl {} 0".format(i)
            sendSerial(cmdStr)
        sendSerial('qcl off')
        sendSerial('dcdc 4 1')  # disable QCL drive
        sendSerial('coil 33000')
    sendSerial('volts', 2)
    print('END OF SEQUENCE', flush=True)


def runRedPitaya():
    print('============ Red Pitaya =============')
    sendSerial('dcdc 8 0')
    time.sleep(20)
    outputStr = subprocess.run(
        ['ssh', 'ckulesa@192.168.1.61', './run_RedPitaya.sh'], stdout=subprocess.PIPE).stdout.decode('utf-8')
    print(outputStr)
    time.sleep(10)

    
# some prep
if len(sys.argv) > 1:
    sleeptime = sys.argv[1]
else:
    sleeptime = 60

loop = 0

# main loop, finally

while True:
    if(os.path.exists("/root/cold") == True):
        cold = 1
    else:
        if(loop > 0):
            if(cold == 1):  # we changed state
                loop = 0  # reset the loop so we startup properly
        cold = 0
    if(os.path.exists("/root/noLO") == True):
        noLO = 1
    else:
        noLO = 0

    with open("/root/limits", "r") as file:
        line = file.readline()
        limits = line.split(',')
        print('LO: ', int(limits[0]), '\t', 'HI: ', int(limits[1]), flush=True)

    print('CPU temp: ', subprocess.run(
        ['cat', '/sys/class/thermal/thermal_zone0/temp'], stdout=subprocess.PIPE).stdout.decode('utf-8'))

    ser = serial.Serial('/dev/ttyACM0', 19200, timeout=0.5)
    print('Loop #: ', loop, ' TIME: ', int(time.time()), ' ', time.ctime(), flush=True)
    if(loop == 0):
        sendSerial('psat 1')    # minimal PSAT
        sendSerial('dcdc 1 0')  # enable PSAT
        sendSerial('mult')      # dummy read
        print('Setting up USB power meter: ', subprocess.run(
            ['ssh', '192.168.1.61', '/home/ckulesa/src/setup_usbpowermeter.pl'], stdout=subprocess.PIPE).stdout.decode('utf-8'))

    if(cold == 1 or loop == 0):   # turn them all on!
        sendSerial('dcdc 2 0', 0.5)  # enable hittite synth +12
        sendSerial('dcdc 6 0', 0.5)  # enable hittite synth +5.5
        sendSerial('dcdc 4 0', 1)  # enable QCL drive
        time.sleep(5)
        sendSerial('setsynth init')
        sendSerial('setsynth freq 12355.5')
        time.sleep(5)
        # check frequency 
        outputStr=subprocess.run(
            ['ssh', '192.168.1.61', '/home/ckulesa/src/run_counter.pl'], stdout=subprocess.PIPE).stdout.decode('utf-8')
        if(float(outputStr) > 1.237e10):
            print('STOP. Frequency too high: ',
                  float(outputStr) / 1e6, ' MHz\n')
            break
        elif(float(outputStr) < 1.234e10):
            print('STOP. Frequency too low: ',
                  float(outputStr) / 1e6, ' MHz\n')
            break
        else:
            print('Frequency OK: ', float(outputStr) / 1e6), ' MHz \n'
    if(noLO == 0):
        runSLED()
    else:
        print('noLO is set.  Skipping SLED bias.')
        runMult()
        outputStr = subprocess.run(
            ['ssh', '192.168.1.61', '/home/ckulesa/src/run_usbpowermeter.pl'], stdout=subprocess.PIPE).stdout.decode('utf-8')
        print('IF Pwr = ', float(outputStr))

    runQCL()        
    runRedPitaya()
    runIsense()
    powerDown(5)

    ser.close()
    time.sleep(float(sleeptime))
    loop += 1
